import pytest
import numpy as np
from rq_interfaces.msg import ProjectionGeometry
from rq_controller.common import PyProjectionGeometry  


@pytest.fixture
def example_message():
    msg = ProjectionGeometry()
    msg.focal_spot_postion_mm.x = 10.
    msg.focal_spot_postion_mm.y = 20.
    msg.focal_spot_postion_mm.z = 30.
    msg.detector_postion_mm.x = 40.
    msg.detector_postion_mm.y = 50.
    msg.detector_postion_mm.z = 60.
    msg.detector_orientation_quad.x = 0.
    msg.detector_orientation_quad.y = 0.
    msg.detector_orientation_quad.z = 0.
    msg.detector_orientation_quad.w = 1.
    msg.focal_spot_orientation_quad.x = 0.
    msg.focal_spot_orientation_quad.y = 0.
    msg.focal_spot_orientation_quad.z = 0.
    msg.focal_spot_orientation_quad.w = 1.
    msg.header.frame_id = "test_frame"
    return msg

def test_initialization():
    focal_spot_mm = np.array([1.0, 2.0, 3.0])
    detector_postion_mm = np.array([4.0, 5.0, 6.0])
    detector_orientation_quad = np.array([0.0, 0.0, 0.0, 1.0])
    frame_id = "test_frame"
    focal_spot_orientation_quad = np.array([0.0, 0.0, 0.0, 1.0])

    geometry = PyProjectionGeometry(focal_spot_mm, detector_postion_mm, detector_orientation_quad, frame_id, focal_spot_orientation_quad)

    assert np.array_equal(geometry.focal_spot_mm, focal_spot_mm)
    assert np.array_equal(geometry.detector_postion_mm, detector_postion_mm)
    assert np.array_equal(geometry.detector_orientation_quad, detector_orientation_quad)
    assert np.array_equal(geometry.focal_spot_orientation_quad, focal_spot_orientation_quad)
    assert geometry.frame_id == frame_id

def test_dummy_method():
    geometry = PyProjectionGeometry.dummy()

    assert np.array_equal(geometry.focal_spot_mm, np.array([1.0, 0.0, 0.0]))
    assert np.array_equal(geometry.detector_postion_mm, np.array([-1.0, 0.0, 0.0]))
    assert np.array_equal(geometry.detector_orientation_quad, np.array([0.0, 0.0, 0.0, 1.0]))
    assert geometry.frame_id == "object"
    assert np.array_equal(geometry.focal_spot_orientation_quad, np.array([0.0, 0.0, 0.0, 1.0]))

def test_from_message(example_message):
    geometry = PyProjectionGeometry.from_message(example_message)

    assert np.array_equal(geometry.focal_spot_mm, np.array([10, 20, 30]))
    assert np.array_equal(geometry.detector_postion_mm, np.array([40, 50, 60]))
    assert np.array_equal(geometry.detector_orientation_quad, np.array([0, 0, 0, 1]))
    assert np.array_equal(geometry.focal_spot_orientation_quad, np.array([0, 0, 0, 1]))
    assert geometry.frame_id == "test_frame"

def test_as_message(example_message):
    geometry = PyProjectionGeometry.from_message(example_message)
    msg = geometry.as_message()

    assert msg.focal_spot_postion_mm.x == 10
    assert msg.focal_spot_postion_mm.y == 20
    assert msg.focal_spot_postion_mm.z == 30
    assert msg.detector_postion_mm.x == 40
    assert msg.detector_postion_mm.y == 50
    assert msg.detector_postion_mm.z == 60
    assert msg.detector_orientation_quad.x == 0
    assert msg.detector_orientation_quad.y == 0
    assert msg.detector_orientation_quad.z == 0
    assert msg.detector_orientation_quad.w == 1
    assert msg.focal_spot_orientation_quad.x == 0
    assert msg.focal_spot_orientation_quad.y == 0
    assert msg.focal_spot_orientation_quad.z == 0
    assert msg.focal_spot_orientation_quad.w == 1
    assert msg.header.frame_id == "test_frame"
